;********************************************************************
;	HEXIN.ASM - a library of hexadecimal input & conversion functions
;********************************************************************

;---Error messages
er$BADCHR	equ 	1			; bad character encountered
er$OVRFLW	equ 	2			; too many digits encountered

;---ASCII characters
TAB 		equ 	9			; Horizontal TAB character
CR			equ 	13			; Carriage Return
LF			equ 	10			; Line feed

.model small
.code

;--------------------------------------------------------------------
;	hexHelp - read a collection of digits into a double word
;
;	INP:	CX	  - maximum number of digits to read (up to 8)
;			DI	  - buffer holding the digits to read
;	OUT:	DX:AX - resulting binary value (or error code if CY set)
;			Carry - SET if error (i.e. no nybbles converted)
;			DI	  - points to next character in buffer
;
hexHelper proc near private uses BX

	; Prepare the buffer for conversion (i.e., skip spaces & zeros)

	assume di:ptr byte			; skip leading spaces
	.while [di]==' ' || [di]==TAB || [di]==CR || [di]==LF
		inc di
	.endw
	xor ch, ch					; no zeros found yet
	.while [di]=='0'			; skip leading zeroes
		inc di
		mov ch,1				; zero found
	.endw

	; Convert the hex digits

	xor bx, bx					; clear the accumulator (DX:BX)
	xor dx, dx
	xor ah, ah					; no digits read yet
	.repeat
	  call hexin_y$n			; get the next nybble
	  .break .if carry?			; stop if bad character
	  repeat 4					; Shift nybble into DX:BX
		shl bx, 1
		rcl dx, 1
	  endm
	  add bl, al
	  inc ah					; Update digit count
	.until 0

	; Set return values

	.if ah==0 && ch==0			; Any hex digits found?
	  mov ax, er$BADCHR 		; N: bad character error
	  stc						;	 signal "ERROR"
	.elseif ah>cl				; Too many digits?
	  mov ax, er$OVRFLW 		; Y: overflow error
	  stc						;	 signal "ERROR"
	.else						; Normal exit ...
	  xchg ax, bx				;	 put LSW in AX
	  clc						;	 signal "NO ERROR"
	.endif
	ret

hexHelper endp


;--------------------------------------------------------------------
;	hexin_y$n - convert the digit pointed to by DI to binary
;
;	INP:	DI - pointer to digit to convert
;	OUT:	AL - holds binary value (0000xxxx)
;			DI - points to next character if no error encountered
;			CY - set on error (i.e., character is not a hex digit)
;
hexin_y$n proc

	mov al, [di]			; Get character
	cmp al, '0' 			; In range '0'..'9'?
	jl hexin_y_Error		; ERR - too low
	cmp al, '9'
	jle hexin_y_Good1		; yep ... exit
	cmp al, 'A' 			; In range 'A'..'F'?
	jl hexin_y_Error		; ERR - too low
	cmp al, 'F'
	jle hexin_y_Good2		; yep ... exit
	cmp al, 'a' 			; In range 'a'..'f'?
	jl hexin_y_Error		; ERR - too low
	cmp al, 'f'
	jle hexin_y_Good3		; yep ... exit
	; ERR - too high

hexin_y_Error:
	stc 					; ERROR-bad character
	ret

hexin_y_Good3:
	sub al, 'a'-'A' 		; Adjust to 'A'..'F'
hexin_y_Good2:
	sub al, 'A'-'9'-1		; Adjust to '9'+1 .. '9'+6
hexin_y_Good1:
	sub al, '0' 			; Adjust to 0..16
	clc 					; NO ERROR
	inc di					; point to next char in buff
	ret

hexin_y$n endp


;--------------------------------------------------------------------
;	hexin_b$n - read in a hexadecimal byte pointed to by DI
;
;	INP:	DI - pointer to text to convert to a hex byte
;	OUT:	AX - contains the resulting byte (AH=0, AL=byte )
;			CY - SET if error
;			DI - points to next byte to read
;
hexin_b$n proc uses CX DX

	mov 	cl, 2				; convert up to 2 digits to HEX
	call	hexHelper
	ret

hexin_b$n endp


;--------------------------------------------------------------------
;	hexin_w$n - read in a hexadecimal word pointed to by DI
;
;	INP:	DI - pointer to text to convert to a hex word
;	OUT:	AX - holds the resulting word
;			CY - SET if error
;			DI - points to next byte to process
;
hexin_w$n proc uses CX DX

	mov 	cl, 4				; convert up to 4 digits to HEX
	call	hexHelper
	ret

hexin_w$n endp


;--------------------------------------------------------------------
;	hexin_d$n - convert a double from text at DI to DX:AX
;
;	INP:	DI	  - pointer to text to convert
;	OUT:	DX:AX - holds the resulting double word
;			CY	  - SET if error
;			DI	  - points to next character in text buffer
;
hexin_d$n proc uses CX

	mov 	cl, 8				; get up to 8 hex digits
	call	hexHelper
	ret

hexin_d$n endp

end
